#version 430 core
/*
* https://learnopengl.com/code_viewer_gh.php?code=src/3.model_loading/1.model_loading/1.model_loading.fs
*/

layout (location = 0) out vec4 FragColor;
layout (location = 1) out vec4 BrightColor;
uniform float brightnessMultiplier;

in vec2 TexCoords;
in vec3 position_world;

uniform sampler2D texture_diffuse1;
layout (binding = 2) uniform samplerCube shadowMap;
//uniform samplerCube shadowMap;
uniform vec3 pointLightPos;
uniform vec3 camera_world;


//taken from learnOpengl.com
vec3 sampleOffsetDirections[20] = vec3[](
   vec3( 1,  1,  1), vec3( 1, -1,  1), vec3(-1, -1,  1), vec3(-1,  1,  1), 
   vec3( 1,  1, -1), vec3( 1, -1, -1), vec3(-1, -1, -1), vec3(-1,  1, -1),
   vec3( 1,  1,  0), vec3( 1, -1,  0), vec3(-1, -1,  0), vec3(-1,  1,  0),
   vec3( 1,  0,  1), vec3(-1,  0,  1), vec3( 1,  0, -1), vec3(-1,  0, -1),
   vec3( 0,  1,  1), vec3( 0, -1,  1), vec3( 0, -1, -1), vec3( 0,  1, -1)
);   

float ShadowCalculation(vec4 fragPosWorldSpace) {
	vec3 fragToLight = fragPosWorldSpace.xyz - pointLightPos;
	float currentDepth = length(fragToLight);
	float closestDepth = texture(shadowMap, fragToLight).r;
	vec3 texSample = fragToLight;
	
	float viewDistance = length(camera_world - fragPosWorldSpace.xyz);

	float bias = 0.15;

	float shadow = 0.0;
	vec2 texelSize = 1.0 / textureSize(shadowMap, 0);
	float sampleScale = (1.0 + viewDistance/100.0) / 25.0; //check?

	for (int i = 0; i < 20; i++) {
		texSample = fragToLight + sampleOffsetDirections[i] * sampleScale;

		if (abs(texSample.y) <= abs(texSample.z) || abs(texSample.y) <= abs(texSample.x)) {
			texSample.y = -texSample.y;
		} else {
			texSample.z = -texSample.z;
		}

		closestDepth = texture(shadowMap, texSample).r;
		//closestDepth = texture(shadowMap, fragToLight).r;
		//float pcfDepth = texture(shadowMap, projCoords.xy + vec2(i, j) * texelSize).r;
		shadow += currentDepth - bias > closestDepth ? 1.0 : 0.0;
	}
	
	shadow /= 20.0;

	return shadow;
}




void main()
{    
    vec3 ambient = vec3(0.2, 0.2, 0.3);
    vec3 pointLightContrib = vec3(0.4, 0.4, 0.5) * (1.0 - ShadowCalculation(vec4(position_world, 1.0))); 
    vec3 textureColor = texture(texture_diffuse1, TexCoords).rgb;
    vec3 lightContrib = ambient + pointLightContrib;
    FragColor = vec4(textureColor * brightnessMultiplier * lightContrib, 1);
	BrightColor = vec4(0.0, 0.0, 0.0, 0.0);
}