#version 430 core
/*
* Copyright 2019 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

layout(location = 0) in vec3 position;

out VertexData {
	vec3 position_world;
} vert;

uniform mat4 modelMatrix;
uniform mat4 viewProjMatrix;
uniform float time;

vec3 calcPosition() {
	//hashing instanceID
	int x = gl_InstanceID * 123456789;
	int y = gl_InstanceID * 362436069;
	int z = gl_InstanceID * 521288629;

	int t = x ^ (x << 11);
	x = y;
	y = z;
	z = gl_InstanceID * (z ^ (z >> 19) ^ (t ^ (t >> 8)));

	float r = fract(float(x) / 4294967296.0);
	float g = fract(float(y) / 4294967296.0);
	float b = fract(float(z) / 4294967296.0);

	r = r * 200.0 - 100.0;
	g = (g * 111.0) + time * 7.0;
	g = g - floor(g / 111) * 111;
	b = b * 90.0 + 10.0;

	return vec3(r, g, b);
}

void main() {
	vec3 offset = calcPosition();

	vec4 position_world_ = modelMatrix * vec4(position - offset, 1);
	vert.position_world = position_world_.xyz ;
	gl_Position = viewProjMatrix * position_world_;
}

