#version 430 core
/*
* this implementation is based on jayconrod.com
*/

layout (location = 0) out vec4 FragColor;
layout (location = 1) out vec4 BrightColor;

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
	vec4 FragPosLightSpace;
} vert;

out vec4 color;

uniform vec3 camera_world;

//layout (binding = 0) uniform sampler2D diffuseTexture;
//layout (binding = 1) uniform sampler2D specularTexture;
layout (binding = 2) uniform samplerCube shadowMap;
layout (binding = 0) uniform samplerCube skyBox;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
uniform float brightnessMultiplier;
uniform float time;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL;

//taken from learnOpengl.com
vec3 sampleOffsetDirections[20] = vec3[](
   vec3( 1,  1,  1), vec3( 1, -1,  1), vec3(-1, -1,  1), vec3(-1,  1,  1), 
   vec3( 1,  1, -1), vec3( 1, -1, -1), vec3(-1, -1, -1), vec3(-1,  1, -1),
   vec3( 1,  1,  0), vec3( 1, -1,  0), vec3(-1, -1,  0), vec3(-1,  1,  0),
   vec3( 1,  0,  1), vec3(-1,  0,  1), vec3( 1,  0, -1), vec3(-1,  0, -1),
   vec3( 0,  1,  1), vec3( 0, -1,  1), vec3( 0, -1, -1), vec3( 0,  1, -1)
);   

float ShadowCalculation(vec4 fragPosWorldSpace) {
	vec3 fragToLight = fragPosWorldSpace.xyz - pointL.position;
	float currentDepth = length(fragToLight);
	float closestDepth = texture(shadowMap, fragToLight).r;
	vec3 texSample = fragToLight;
	
	float viewDistance = length(camera_world - fragPosWorldSpace.xyz);

	float bias = 0.15;

	float shadow = 0.0;
	vec2 texelSize = 1.0 / textureSize(shadowMap, 0);
	float sampleScale = (1.0 + viewDistance/100.0) / 25.0; //check?

	for (int i = 0; i < 20; i++) {
		texSample = fragToLight + sampleOffsetDirections[i] * sampleScale;

		if (abs(texSample.y) <= abs(texSample.z) || abs(texSample.y) <= abs(texSample.x)) {
			texSample.y = -texSample.y;
		} else {
			texSample.z = -texSample.z;
		}

		closestDepth = texture(shadowMap, texSample).r;
		shadow += currentDepth - bias > closestDepth ? 1.0 : 0.0;
	}
	
	shadow /= 20.0;

	return shadow;
}


vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, vec3 specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);

	vec3 diffuse = diffuseF * diffuseC * max(0, dot(n, l));
	vec3 specular = specularF * specularC * pow(max(0, dot(r, v)), alpha);

	return ((diffuse + specular) * att);
	
	// return ((diffuseF * diffuseC * max(0, dot(n, l)) + specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att); 
}

const float pi = 3.14159;
float amplitude[] = {0.5, 0.125, 0.8};
float wavelength[] = {4 * pi, 2 * pi, 8 * pi};
float speed[] = {1.0, 6.0, 0.5};
vec2 direction[] = { vec2(1.0, 0.0), normalize(vec2(1.0, 1.0)), normalize(vec2(1.0, -1.0)) }; // x is x, y is z coordinate in this case

//height function omitted, as the position is already set

//deriviations (tangent and bitangent)

float dWavedx(int index) {
	float frequency = 2 * pi / wavelength[index];
	float phase = speed[index] * frequency;
	float theta = dot(direction[index], vec2(vert.position_world.x, vert.position_world.z));
	float A = amplitude[index] * direction[index].x * frequency;
	return A * cos(theta * frequency + time * phase);
}

float dWavedy(int index) {
	float frequency = 2 * pi / wavelength[index];
	float phase = speed[index] * frequency;
	float theta = dot(direction[index], vec2(vert.position_world.x, vert.position_world.z));
	float A = amplitude[index] * direction[index].y * frequency;
	return A * cos(theta * frequency + time * phase);
}

vec3 calculateNormalizedWaveNormal() {
	//vec3 nOut = vec3(0.0, 1.0, 0.0);

	float dx = 0.0;
	float dy = 0.0;

	dx += dWavedx(0);
	dy += dWavedy(0);

	dx += dWavedx(1);
	dy += dWavedy(1);

	dx += dWavedx(2);
	dy += dWavedy(2);

	vec3 nOut = vec3(-dx, 1.0, -dy);

	return normalize(nOut);
}

void main() {	
	//vec3 n = normalize(vert.normal_world);
	vec3 n = calculateNormalizedWaveNormal();
	vec3 v = normalize(camera_world - vert.position_world);

	//vec3 materialSpecular = texture(specularTexture, vert.uv).rgb *3;
	vec3 materialSpecular = vec3(1.0, 1.0, 1.0);
	
	//vec3 texColor = texture(diffuseTexture, vert.uv).rgb;
	vec4 color = vec4(0.0, 0.0, 0.02, 1.0);

	//color = vec4(texColor * materialCoefficients.x, 1); // ambient
	vec3 r = reflect(-v, n);
	vec3 texColor = texture(skyBox, r).rgb * vec3(0.5, 0.5, 0.8); 
	//vec4 color = vec4(texColor, 1.0) * vec4(0.0, 0.0, 0.5, 1.0);

	//shadow
	float shadow = ShadowCalculation(vec4(vert.position_world, 1.0)); 
	
	// add directional light contribution
	texColor.rgb += phong(n, -dirL.direction, v, dirL.color * texColor, materialCoefficients.y, dirL.color, materialSpecular, specularAlpha, false, vec3(0));
	// color.rgb += phong(n, -dirL.direction, v, dirL.color * texColor, (1.0 - shadow) * materialCoefficients.y, dirL.color, (1.0 - shadow) * materialSpecular, specularAlpha, false, vec3(0));
			
	// add point light contribution
	texColor.rgb += phong(n, pointL.position - vert.position_world, v, pointL.color * texColor, (1.0 - shadow) * materialCoefficients.y, pointL.color, (1.0 - shadow) * materialSpecular, specularAlpha, true, pointL.attenuation);
	// color.rgb += phong(n, pointL.position - vert.position_world, v, pointL.color * texColor, materialCoefficients.y, pointL.color, materialSpecular, specularAlpha, true, pointL.attenuation);

        color = vec4(brightnessMultiplier * color.rgb, 1);

	FragColor = vec4(texColor, 1.0);
	BrightColor = (color.r >= 1.0 && color.g >= 1.0 && color.b >= 1.0) ? vec4(1.0) : vec4(0.0);
}

